Attribute VB_Name = "InitMod"
Option Explicit

Public DX As DirectX8
Public D3D As Direct3D8
Public D3DDevice As Direct3DDevice8
Public D3DX As New D3DX8

Public matView As D3DMATRIX
Public matWorld As D3DMATRIX
Public matProj As D3DMATRIX

Public DispMode As D3DDISPLAYMODE
Public D3DWindow As D3DPRESENT_PARAMETERS

Public Running As Boolean

Public Lights() As D3DLIGHT8

Public Const RadPerDeg As Single = 1.74532925199433E-02 'this means there are 1.7~~E-02 degrees per radian
Public Const DegPerRad As Single = 57.2957795130823     'this meand there are 57.2~~ radians per degree

Public Const PI As Single = 3.14159265358979
Public Const D_1  As Single = PI / 180                  '1   degrees in radians
Public Const D_90 As Single = PI / 2                    '90  degrees in radians
Public Const D_180 As Single = PI                       '180 degrees in radians
Public Const D_270 As Single = (PI / 2) * 3             '270 degrees in radians
Public Const D_360 As Single = PI * 2                   '360 degrees in radians

Public Const TSPEED = D_1 + 0.06    'Turning Speed
Public Const MSPEED = 2             'Moving Speed

Public Angle, ConvAngle As Single   'Holds the current angle
Public CAMX, CAMY, CAMZ As Single   'hold the position of the camera on the x, y and z axis

Public Type MyMesh
    Matrix As D3DMATRIX
    Mesh As D3DXMesh            'the actual 3d object
    Mat() As D3DMATERIAL8       'array to store the materials used in the 3d object
    Tex() As Direct3DTexture8   'array to store the textures used in the 3d object
    MCount As Long              'the number of materials used in the 3d object
    MX As Single                'holds the current mesh x
    MY As Single                'holds the current mesh y
    MZ As Single                'holds the current mesh z
    MAngle As Single            'the current angle of the mesh
End Type

Public TreeMesh() As MyMesh
Public HouseMesh() As MyMesh
Public RoadMesh() As MyMesh

Public SnowMesh(0) As MyMesh    'cause there only one snowman and world mesh we can
Public WorldMesh(0) As MyMesh   'use a static array of 1 valur

Public MtrlBuffer As D3DXBuffer

'=============<FONT>===================
Public D3DFont As D3DXFont      'the next 4 variable are used for font
Public FontDesc As IFont
Public TextRect(1) As RECT
Public TempFont As New StdFont
Public Function DTR(Deg As Single) As Single    'this function takes a degree and converts it into a radian value
    DTR = Deg * DegPerRad
End Function
Public Function RTD(Rad As Single) As Single    'the same as DTR but around the other way
    RTD = Rad * RadPerDeg
End Function
Private Function SetDispMode()
    DispMode.Format = D3DFMT_R5G6B5
    DispMode.Width = 1024    'set this to what ever screen size you want
    DispMode.Height = 768    'i just prefer 1024-768 so i don't have to resize any windows after the code crashing
    D3DWindow.SwapEffect = D3DSWAPEFFECT_FLIP
    D3DWindow.Windowed = 0   'using full screen mode
    D3DWindow.BackBufferCount = 1
    D3DWindow.BackBufferFormat = D3DFMT_R5G6B5
    D3DWindow.BackBufferWidth = 1024
    D3DWindow.BackBufferHeight = 768
    D3DWindow.hDeviceWindow = frmMain.hWnd
    D3DWindow.EnableAutoDepthStencil = 1
    D3DWindow.AutoDepthStencilFormat = D3DFMT_D16
End Function
'=================================================================================='
Public Function InitDX() As Boolean
    On Error GoTo BailOut
    Set DX = New DirectX8
    Set D3D = DX.Direct3DCreate()
    Set D3DX = New D3DX8
    
    SetDispMode
    
    'this line creates our rendering device
    Set D3DDevice = D3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, frmMain.hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, D3DWindow)
    
    D3DDevice.SetRenderState D3DRS_LIGHTING, 1          'enable lighting
    D3DDevice.SetRenderState D3DRS_ZENABLE, 1           'enable the z buffer
    D3DDevice.SetRenderState D3DRS_AMBIENT, &HDDDDDD    'this basically means lighting for the whole screen - change it and see what happens
    'if your not sure what ambient lighting does just comment the line above and
    'uncomment the line below
    'D3DDevice.SetRenderState D3DRS_AMBIENT, &H202020
   
    D3DXMatrixIdentity matView
    D3DXMatrixLookAtLH matView, MakeVector(0, -D_360, D_1), MakeVector(0, 0, 0), MakeVector(0, 1, 0)
    D3DDevice.SetTransform D3DTS_WORLD, matView
    
    D3DXMatrixPerspectiveFovLH matProj, PI / 4, PI * 0.3, 5, 1000
    D3DDevice.SetTransform D3DTS_PROJECTION, matProj
    
    'create a single landscape and snowman
    CreateMesh "\X_Files\LandScape.x", WorldMesh(0).Mesh, WorldMesh(0).Mat, WorldMesh(0).Tex, WorldMesh(0).MCount, 1024, 512
    CreateMesh "\X_Files\SnowMan.x", SnowMesh(0).Mesh, SnowMesh(0).Mat, SnowMesh(0).Tex, SnowMesh(0).MCount, 256, 256
    
    MyReadMeshFromX "\LoadFiles\LoadX01.txt", TreeMesh()  'load multiple Tree mesh's
    MyReadMeshFromX "\LoadFiles\LoadX02.txt", HouseMesh() 'load multiple House mesh's
    MyReadMeshFromX "\LoadFiles\LoadX03.txt", RoadMesh()  'load multiple Road mesh's
    
    TempFont.Name = "Times New Roman"
    TempFont.Size = 8
    Set FontDesc = TempFont
    
    'call basic initialize functions the names pretty much give that away
    InitLights
    InitKeyBoard
    
    Set D3DFont = D3DX.CreateFont(D3DDevice, FontDesc.hFont)
    
    Running = True
    
    InitDX = True
    Exit Function
BailOut:
    InitDX = False
End Function
'=================================================================================='
Private Function InitLights()
    On Error GoTo BailOut
    Dim Mtrl As D3DMATERIAL8 'Material
    Dim Col As D3DCOLORVALUE 'Color
    'Create color
    Col.a = 1: Col.r = 1: Col.g = 1: Col.b = 1
    'Apply material
    Mtrl.Ambient = Col: Mtrl.diffuse = Col
    D3DDevice.SetMaterial Mtrl
    
    'Create directional light
    Lights(0).Type = D3DLIGHT_DIRECTIONAL
    Lights(0).diffuse.r = 1
    Lights(0).diffuse.g = 1
    Lights(0).diffuse.b = 1
    Lights(0).Direction = MakeVector(0, 0, 50)

    Lights(1).Type = D3DLIGHT_POINT
    Lights(1).Position = MakeVector(0, -10, 6)
    Lights(1).diffuse.b = 1
    Lights(1).Range = 1000
    Lights(1).Attenuation1 = 0.005  'Set the Linear Attenuation to 0.05
    
    Lights(2).Type = D3DLIGHT_SPOT
    Lights(2).Position = MakeVector(0, D_360, -D_1)
    Lights(2).Range = 500
    Lights(2).Direction = MakeVector(0, 0, -D_1) 'Along the X axis
    Lights(2).Theta = 15 * D_1 'Inner Cone
    Lights(2).Phi = 25 * D_1 'Outer Cone
    Lights(2).diffuse.g = 1
    Lights(2).Attenuation1 = 0.1
    
    'Apply lights to device
    D3DDevice.SetLight 0, Lights(0)
    D3DDevice.SetLight 1, Lights(1)
    D3DDevice.SetLight 2, Lights(2)
    
    InitLights = True
    Exit Function
BailOut:
    InitLights = False
End Function
'=================================================================================='
Private Function InitKeyBoard() As Boolean
On Local Error GoTo BailOut:
    
    Set DKI = DX.DirectInputCreate
    'the next line creates our keyboard device the string "GUID_SysKeyboard" is
    'not just a random string it tells the creator what to create
    Set DKIDevice = DKI.CreateDevice("GUID_SysKeyboard")
    
    DKIDevice.SetCommonDataFormat DIFORMAT_KEYBOARD
    DKIDevice.SetCooperativeLevel frmMain.hWnd, DISCL_BACKGROUND Or DISCL_NONEXCLUSIVE
        
    'set up the buffer...
    DevProp.lHow = DIPH_DEVICE
    DevProp.lData = 10
    DKIDevice.SetProperty DIPROP_BUFFERSIZE, DevProp
    
    'basically now that we have created the keyboard device we need to make it work
    DKIDevice.Acquire   'tell it to start working
    
    InitKeyBoard = True
    Exit Function
BailOut:
    InitKeyBoard = False
End Function
'=================================================================================='
Public Function CreateMesh(FileName As String, Mesh As D3DXMesh, MeshMaterials() As D3DMATERIAL8, MeshTextures() As Direct3DTexture8, nMaterials As Long, SizeX As Integer, SizeY As Integer)
Dim TextureName As String
Dim q As Integer
    'get the information out of the .x file and store it in mesh
    Set Mesh = D3DX.LoadMeshFromX(App.Path & FileName, D3DXMESH_MANAGED, D3DDevice, Nothing, MtrlBuffer, nMaterials)
    
    'resize the materials and texture array to the amount of object in the 3d object
    'eg. if you have a house, it will have a walls object, a door object etc
    ReDim MeshMaterials(nMaterials - 1) As D3DMATERIAL8
    ReDim MeshTextures(nMaterials - 1) As Direct3DTexture8
    
    'for each of the 3d object, objects run through and set its ambient lighting
    'and if it had a texture when you created it then give the object its texture
    For q = 0 To nMaterials - 1
        'get the the 3d objects, object
        D3DX.BufferGetMaterial MtrlBuffer, q, MeshMaterials(q)
        'set its abient lighting value
        MeshMaterials(q).Ambient = MeshMaterials(q).diffuse
        'try to find a texture for the object
        TextureName = D3DX.BufferGetTextureName(MtrlBuffer, q)
        If TextureName <> "" Then
            'if it has one then create it - the "SizeX, SizeY," parts of this call are
            'for the size of the texture you want to map
            Set MeshTextures(q) = D3DX.CreateTextureFromFileEx(D3DDevice, App.Path & "\Textures\" & TextureName, SizeX, SizeY, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR, 0, ByVal 0, ByVal 0)
        End If
    Next q
    'clear the buffer that is used to store the 3d objects, object
    Set MtrlBuffer = Nothing
    
End Function
'==================================================================================
'this is a simple little function to read the multimesh files and create them
'this function was inspired by simon prices EX-Demo
Public Function MyReadMeshFromX(FileName As String, InMesh() As MyMesh)
    Dim InText, XFileName As String, FileNum, LoopCount, MeshNum As Integer, IX, IY, IZ As Long
    FileNum = FreeFile
    Open App.Path + FileName For Input As FileNum
        Do Until EOF(FileNum)
            Line Input #FileNum, InText
            If (Not (InText = "") Or (InText = ";")) Then   'so if its not a blank line or a comment line carry on
                Select Case InText
                Case "<FileName>"
                    Input #FileNum, XFileName   'retrive the .x file name
                Case "<Location>"
                    Input #FileNum, LoopCount   'find how many objects are going to be created
                    For MeshNum = 0 To LoopCount
                        Input #FileNum, IX, IY, IZ      'get the new x,y,z positions
                        ReDim Preserve InMesh(MeshNum)
                        InMesh(MeshNum).MX = IX
                        InMesh(MeshNum).MY = IY
                        InMesh(MeshNum).MZ = IZ
                        'create the new mesh
                        CreateMesh XFileName, InMesh(MeshNum).Mesh, InMesh(MeshNum).Mat, InMesh(MeshNum).Tex, InMesh(MeshNum).MCount, 512, 512
                    Next MeshNum
                End Select
            End If
        Loop
    Close FileNum
End Function
'==================================================================================
Public Function MakeVector(X As Single, Y As Single, Z As Single) As D3DVECTOR
    MakeVector.X = X
    MakeVector.Y = Y
    MakeVector.Z = Z
End Function
'==================================================================================
